// STGATexture.cpp: Implementierung der Klasse STGATexture.
//
// Copyright by Andr Stein
// E-Mail: andre_stein@gmx.de, andre_stein@web.de
// http://www.steinsoft.net
//////////////////////////////////////////////////////////////////////

#include "stgatexture.h"
#include <iostream>
#include <stdio.h>

using namespace std;

//Defines
#ifndef GL_CLAMP_TO_EDGE
        #define GL_CLAMP_TO_EDGE  0x812F
#endif


//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

STGATexture::STGATexture()
{
	texture = 0;
}

STGATexture::~STGATexture()
{
	destroy();
}

STGATexture::STGATexture(const char* filename,int minFilter,int magFilter,
				   bool clampToEdge)
{
	createTexture(filename,minFilter,magFilter,clampToEdge);
}


///////////////////////////////
// Funktionen
///////////////////////////////

bool STGATexture::createTexture(const char *filename, int minFilter, int magFilter,
				   bool clampToEdge)
{
        STGA tgaFile;

 	destroy();

	if (loadTGA(filename,tgaFile))
	{
		glPixelStorei(GL_UNPACK_ALIGNMENT,1);

		glGenTextures(1,&texture);
		glBindTexture(GL_TEXTURE_2D,texture);

		if (clampToEdge)
		{
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
		} else {
			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		}

		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,minFilter);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,magFilter);
		glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

           	glTexImage2D(GL_TEXTURE_2D,0,tgaFile.byteCount,tgaFile.width,
			     tgaFile.width,0,(tgaFile.byteCount == 3 ? GL_BGR : GL_BGRA),
			GL_UNSIGNED_BYTE,tgaFile.data);

		return true;
	}

	return false;
}

bool STGATexture::createMipmap(const char *filename, int minFilter, int magFilter,
				   bool clampToEdge)
{
        STGA tgaFile;

	destroy();

        if (loadTGA(filename,tgaFile))
	{
		glPixelStorei(GL_UNPACK_ALIGNMENT,1);

		glGenTextures(1,&texture);
		glBindTexture(GL_TEXTURE_2D,texture);

		if (clampToEdge)
		{
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
		} else {
			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		}

		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,minFilter);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,magFilter);
		glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

		gluBuild2DMipmaps(GL_TEXTURE_2D,tgaFile.byteCount,tgaFile.width,
				  tgaFile.height,(tgaFile.byteCount == 3 ? GL_BGR : GL_BGRA),
			GL_UNSIGNED_BYTE,tgaFile.data);

        	return true;
	}

	return false;
}

GLuint STGATexture::getGLTexture()
{
	return texture;
}

void STGATexture::select()
{
	glBindTexture(GL_TEXTURE_2D,texture);
}

void STGATexture::destroy()
{
	if (texture)
	{
		glDeleteTextures(1,&texture);
		texture = 0;
        }
}

bool STGATexture::loadTGA(const char *filename, STGA& tgaFile)
{
	FILE *file;
	unsigned char type[4];
	unsigned char info[6];

        file = fopen(filename, "rb");

        if (!file)
		return false;

	fread (&type, sizeof (char), 3, file);
	fseek (file, 12, SEEK_SET);
	fread (&info, sizeof (char), 6, file);

	//image type either 2 (color) or 3 (greyscale)
	if (type[1] != 0 || (type[2] != 2 && type[2] != 3))
	{
		fclose(file);
		return false;
	}

	tgaFile.width = info[0] + info[1] * 256;
	tgaFile.height = info[2] + info[3] * 256;
	tgaFile.byteCount = info[4] / 8;

	if (tgaFile.byteCount != 3 && tgaFile.byteCount != 4) {
		fclose(file);
		return false;
	}

	long imageSize = tgaFile.width * tgaFile.height * tgaFile.width * tgaFile.byteCount;

	//allocate memory for image data
	tgaFile.data = new unsigned char[imageSize];

	//read in image data
	fread(tgaFile.data, sizeof(unsigned char), imageSize, file);

	//close file
	fclose(file);

	return true;
}
